package charite.christo;
import charite.christo.interfaces.*;
import java.io.File;
import java.util.HashSet;
import java.io.IOException;
import java.util.Comparator;

/**
   @author Christoph Gille
   f.toString()==f.toString()
   f.getAbsolutePath()!=f.getAbsolutePath()
   f.getName()==f.getName()
*/
public class ChFile extends File implements Comparator {
    private boolean unknownDriveLetter;
    private static boolean isWin;
    public ChFile(String fn, boolean isWindows) {
        super( (isWin=isWindows) && fn.startsWith("?:") ? "C:"+fn.substring(2) : fn);
        this.unknownDriveLetter=isWin && fn.startsWith("?:");
    }
    public ChFile(File f,String s, boolean isWindows) { super(f,s); ChFile.isWin=isWindows; }
    @Override public String toString(){
        String s=getAbsolutePath();
        return unknownDriveLetter ? "?:"+s.substring(2) : s;
    }
    public int compare(Object o1,Object o2) {
        final File f1= o1 instanceof File ? (File)o1:null;
        final File f2= o2 instanceof File ? (File)o2:null;
        if (f1==null && f2==null) return 0;
        if (f1==null) return 1;
        if (f2==null) return -1;
        final boolean d1=f1.isDirectory(), d2=f2.isDirectory();
        if (d1!=d2) return  d1 ? -1 : 1;
        if (this==comparator['M']) {
            long t1=f1.lastModified(),t2=f2.lastModified();
            return t1<t2 ? -1 : t1>t2 ? 1 : 0;
        }
        if (this==comparator['S']) {
            long s1=f1.length(), s2=f2.length();
            return s1<s2 ? -1 : s1>s2 ? 1 : 0;
        }
        return f1.getName().compareTo(f2.getName());
    }

    private final static Comparator[] comparator=new Comparator['Z'];
    public static Comparator comparator(char type) {
        assert type=='M' || type=='S' || type=='A';
        if (comparator[type]==null) comparator[type]=new ChFile("X",isWin);
        return comparator[type];
    }

    private String name;
    @Override public String getName() {
        if (name==null) name=super.getName();
        return name;
    }

    /* <<<  <<< */
    /* ---------------------------------------- */
    /* >>> getCanonicalPath >>> */
    private String canonicalPath;
    public static String getCanonicalPathWithSlash(File f) {
        if (f==null) return null;
        String s;
        if (f instanceof ChFile && (s=((ChFile)f).canonicalPath)!=null) return s;
        try {
            s=f.getCanonicalPath();
        } catch(IOException iox) { s=f.getAbsolutePath();}
        s=s.replace('\\','/');
        if (f.isDirectory() && !s.endsWith("/")) s=s+"/";
        if (f instanceof ChFile) ((ChFile)f).canonicalPath=s;
        return s;
    }
    /* <<< getCanonicalPath <<< */
    /* ---------------------------------------- */
    /* >>> Security >>> */
    private final static HashSet<String> VIOLATIONS=new HashSet();
    public final static String
        DIR_DesktopUtils="/strap_utils/",
        DIR_STRAP_ALIGN="StrapAlign/",
        DIR_METANNOGEN="/metannogen/",
        ALLOWED_SUBSTRING[]={DIR_DesktopUtils, DIR_STRAP_ALIGN, DIR_METANNOGEN, "location_of_StrapAlign.ini"};

    private static boolean securityEnabled;
    public static void securitySetEnabled(boolean b) {securityEnabled=b;}
    public static boolean renameToIgnoreSecurity(File fSrc, File fDest) { return fSrc!=null && fDest!=null && fSrc.renameTo(fDest);}
    private static String allowedPath;
    public static void securitySetAllowedPath(File dir) {
        if (dir==null || (dir.exists() && !dir.isDirectory())) {
            System.out.println("dir="+dir);
            assert false;
        }
        allowedPath=getCanonicalPathWithSlash(dir);
    }
    static boolean renameFile(File src, File dest) {
        return securityCanDelete(src) && securityCanDelete(dest) ? src.renameTo(dest): false;
    }
    private static boolean securityCanDelete(File f) {
        if (f==null) return false;
        if (!securityEnabled) return true;
        String path=getCanonicalPathWithSlash(f);
        if (ChExec.IS_ENABLED) if (allowedPath!=null && path.startsWith(allowedPath)) return true;
        if (path.endsWith(".jnlp") && path.indexOf("strap")>=0) return true;
        for(String s : ALLOWED_SUBSTRING) if (path.indexOf(s)>=0) return true;
        System.out.println(ChConstants.ANSI_RED+"securityCanDelete failed for "+ChConstants.ANSI_RESET+path);
        VIOLATIONS.add(path);
        return false;
    }

    static boolean delFile(File f) { return securityCanDelete(f) ? f.delete() : false; }

    static  java.io.OutputStream fileOutputStream(File f, boolean append) throws IOException {
        if (f==null) return null;
        if (!securityCanDelete(f)) {
            System.out.println(ChConstants.ANSI_RED+"new FileOutputStream failed for "+ChConstants.ANSI_RESET+f+"\n");
            throw new IOException("ChFile: "+f);
        }
        return new java.io.FileOutputStream(f,append);
    }
    static  java.io.Writer fileWriter(File f) throws IOException {
        if (f==null) return null;
        if (!securityCanDelete(f)) {
            System.out.println(ChConstants.ANSI_RED+"new FileWriter failed for "+ChConstants.ANSI_RESET+f+"\n");
            throw new IOException("ChFile: "+f);
        }
        return new java.io.FileWriter(f);
    }

    public static boolean securityPrintViolations() {
        if (VIOLATIONS.size()>0) {
            System.err.println("ChFile: Violating file accesses:\n"+VIOLATIONS);
            new IllegalAccessException().printStackTrace();
            VIOLATIONS.clear();
            return true;
        }
        return false;
    }
}
