package charite.christo.protein;
import charite.christo.*;
import charite.christo.interfaces.*;
import java.awt.*;
import java.lang.ref.*;
import javax.swing.*;
import java.awt.event.*;
import java.io.File;
import java.util.*;
import static java.lang.System.*;
import static charite.christo.ChUtils.*;
import static charite.christo.ChConstants.*;
public class ChAstex implements ProteinViewer, ChProcessEvent, astex.MoleculeRendererListener {
    /* ---------------------------------------- */
    /* >>> Conversions >>> */
    public astex.MoleculeRenderer getMoleculeRenderer() {
        final astex.MoleculeViewer mv=getAstex();
        return mv!=null ? mv.getMoleculeRenderer() : null;
    }
    private astex.Molecule getMolecule() {
        final astex.MoleculeRenderer mr=getMoleculeRenderer();
        return mr!=null ? mr.getMolecule(this.proteinName) : null;
    }
    private astex.Residue getResidue(int iA) {
        final astex.Molecule mol=getMolecule();
        final Protein p=getProtein();
        if (mol==null || p==null) return null;
        final int  rN=p.getResidueNumber(iA);
        final byte rC=p.getResidueChain(iA);
        final byte rI=p.getResidueInsertionCode(iA);
        for(int iC=mol.getChainCount(); --iC>=0;) {
            final astex.Chain chain=mol.getChain(iC);
            if (rC==0||chrAt(0,chain.getName())==rC) {
                for(int iR=chain.getResidueCount(); --iR>=0;) {
                    final astex.Residue res=chain.getResidue(iR);
                    if (res.getNumber()==rN && (rI==0||res.getInsertionCode()==rI)) return res;
                }
            }
        }
        return null;
    }
    private int toResidueIndex(astex.Atom atom) {
        final astex.Molecule molecule=getMolecule();
        final Protein p=getProtein();
        if (atom==null || p==null || atom.getMolecule()!=molecule) return INT_NAN;
        final astex.Residue residue=atom.getResidue();
        if (residue!=null && residue.isStandardAminoAcid()) {
            final astex.Chain chain = residue.getParent();
            final char chainChar=chrAt(0,chain!=null ? chain.getName() : null);
            return p.residueNumberAndChain2residueIdx(residue.getNumber(),chainChar, (char)residue.getInsertionCode());
        }
        return INT_NAN;
    }
    /* <<< Conversions <<< */
    /* ---------------------------------------- */
    /* >>> Protein >>> */
    private Protein protein;
    private String proteinName;
    public Protein getProtein() { return protein;}
    public boolean setProtein(Protein p, ProteinViewer inSameView, Runnable runAfterInstallation, long options) {
        if (protein!=null) return false;
        final File file=p.getFileMayBeWithSideChains();
        if (file==null) return false;
        this.protein=p;
        this.proteinName=p.getName();
        final ChAstex inSameViewPO= (ChAstex) ( inSameView instanceof ChAstex ? (ChAstex)inSameView : inSameView instanceof ChAstexPROXY ? ((ChAstexPROXY)inSameView).pv() : null);
        astex.MoleculeViewer moleculeViewer=inSameViewPO!=null ? inSameViewPO.getAstex() : null;
        if (moleculeViewer==null) {
            moleculeViewer=new astex.MoleculeViewer();
            moleculeViewer.setApplication(true);
            final JMenuBar mb=this.share_menubar=menuBar2j(moleculeViewer.createMenuBar());
            mb.add(pnl());
            moleculeViewer.setUsePopupMenu(true);
            this.share_refPanel=weakRef(moleculeViewer);
            this.share_vView=new UniqueList(ChAstex.class);
            this.share_ref_astexMV=weakRef(moleculeViewer);
        } else {
            this.share_menubar=inSameViewPO.share_menubar;
            this.share_ref_astexMV=inSameViewPO.share_ref_astexMV;
            this.share_refPanel=inSameViewPO.share_refPanel;
            this.share_vView=inSameViewPO.share_vView;
        }
        getMoleculeRenderer().addMoleculeRendererListener(this);
        adUniq(this, share_vView);
        // getMoleculeRenderer().addMolecule(file.toString(), p.getName());
        final astex.Molecule molecule=astex.MoleculeIO.readPDB(new astex.FILE(ChUtils.inputStream(file.toString(), true)));
        if (molecule==null) return false;
        molecule.setSymmetry(null);
        molecule.setFilename(file.toString());
        molecule.setName(p.getName());
        getMoleculeRenderer().addMolecule(molecule);
        for(String s : getCustomSettings(ChAstexPROXY.class, Customize.CLASS_InitCommands)) interpret(s,null);
        getViewerPanel().addMouseListener(LI);
        getViewerPanel().addMouseWheelListener(LI);
        return true;
    }
    /* <<< Protein <<< */
    /* ---------------------------------------- */
    /* >>> Disposable >>> */
    private boolean _disposed;
    public void dispose() {
        if (!_disposed) {
            _disposed=true;
            final UniqueList v=this.share_vView;
            interpret("molecule remove '"+this.proteinName+"';", (String)null);
            v.remove(this);
            AbstractProxy.disposeProxy(this);
            interpret("object remove \"schematic_PROTEIN\";", (String)null);
            this.share_ref_astexMV=null;
            this.share_menubar=null;
        }
    }
    /* <<< Disposable <<< */
    /* ---------------------------------------- */
    /* >>> CommandInterpreter >>> */
    public void interpret(String command0,String id) {
        //putln("ChAstex#interpret  >"+command0+"<  id="+id+"<");
        final String command=command0.trim();
        final astex.MoleculeRenderer mr=getMoleculeRenderer();
        final astex.Renderer r=mr!=null ? mr.renderer:null;
        final String word0=toStrgIntrn(toStrgTrim(wordAt(0,command)));
        if (0<=idxOfStrg(word0, COMMANDS)) {
            if (word0==INTERPRET_CENTER) {
                final String group=id!=null ? "group '"+id+"'" : "all and aminoacid";
                interpret("center  ( sphere 20 around ( "+group+" ) ); ", (String)null);
            }
            String send=null;
            final String s=
                word0==COMMANDspheres?"spheres":
                word0==COMMANDlines?"lines":
                word0==COMMANDsticks?"sticks":
                null;
            final boolean off=command.endsWith(" off");
            final String onOff=off ? " off":" on ";
            if (s!=null) send="display  "+s+onOff+" (group "+id+");\n";
            if (word0==COMMANDribbons) send=off ? "object remove 'schematic_"+id+"';\n" : "schematic -name 'schematic_"+id+"' -alltube  true  -ribbonwidth 20 all;\n";
            if (word0==COMMANDcolor) send="color "+delPfx(word0,command)+"   (group "+id+");\n";
            if (word0==COMMANDsurface) send=off ? "object remove 'surface_"+id+"';\n" : "surface -solid true  'surface_"+id+"' pink (all);\n";
            if (mr!=null && send!=null) {
                putln(ANSI_CYAN+"Astex>"+ANSI_RESET+send+ANSI_RED+"<"+ANSI_RESET);
                mr.executeInternal(send);
            }
        } else if (id==INTERPRET_ROTATE) {
            final char axis=chrAt(0,command);
            final double f=atof(command,1);
            if (axis=='x') r.rotateX(f);
            if (axis=='y') r.rotateY(f);
            if (axis=='z') r.rotateZ(f);
        } else if (id==INTERPRET_ZOOM) {
            final int zoom=atoi(command);
            r.applyZoom((100.0-zoom)/100);
        } else {
            if (mr!=null && sze(command)>0) {
                final String cmd=rplc(RESIDUES,id, rplc(PROTEIN,this.proteinName,command));
                putln(ANSI_RED+"Astex>"+ANSI_RESET+cmd+ANSI_RED+"<"+ANSI_RESET);
                mr.executeInternal(cmd);
            }
        }
    }
    /* <<< CommandInterpreter <<< */
    /* ---------------------------------------- */
    /* >>> ResidueSelection >>> */
    private final StringBuffer SB=new StringBuffer();
    private  void drawCursor(int iA) {
        final astex.Residue res=getResidue(iA);
        final Component canvas=getViewerPanel();
        final Graphics g=canvas!=null ? canvas.getGraphics() : null;
        if (res!=null && g!=null) {
            for(int i=res.getAtomCount(); --i>=0;){
                final astex.Atom atom=res.getAtom(i);
                if ("CA".equals(atom.getAtomLabel()))    sphere(atom);
            }
        }
    }
    private void sphere(astex.Atom atom) {
        if (atom!=null) {
            final double x=atom.x, y=atom.y, z=atom.z;
            double radius= 5*atom.getVDWRadius();
            final astex.MoleculeRenderer mr=getMoleculeRenderer();
            final astex.Renderer r=mr!=null ? mr.renderer :null;
            if (r!=null) {
                r.drawSphere(x+2*radius,y,z, radius, 0xFF00FF, 0);
                r.drawCylinder(x,y,z,  x+radius,y+radius,z+radius, 0xFF00FF, 0xFF00FF,radius);
                mr.repaint();
            }
        }
    }
    public void setSelectedResidues(final boolean[] bb,final int atomType,final String representation_id) {
        if (representation_id==ProteinViewer.SELECTION_CURSOR) {
            drawCursor(firstTrue(bb));
        }
        {
            final astex.MoleculeRenderer mr=getMoleculeRenderer();
            final Protein p=getProtein();
            if (mr==null || p==null) return;
            final StringBuffer sb=SB;
            sb.setLength(0);
            sb.append("select ( molecule ").append('"').append(this.proteinName).append('"');
            final int rn[]=p.getResidueNumber();
            final byte chains[]=p.getResidueChain();
            char chain=0;
            if (firstTrue(bb)>=0) {
                if (rn!=null) {
                    sb.append(" and residue ");
                    for(int i=mini(rn.length, bb.length); --i>=0;) {
                        if (!bb[i]) continue;
                        sb.append(rn[i]).append(' ');
                        if (chains!=null && chains.length>i) chain=(char)chains[i];
                    }
                    if (chain!=0) sb.append(" and chain '").append(chain).append('\'');
                }
                if (representation_id!=null) {
                    sb.append(" );\ndefine ").append(representation_id).append(" current;");
                }
                interpret(sb.toString() , representation_id);
            }
            mr.repaint();
        }
    }
    /* <<< ResidueSelection <<< */
    /* ---------------------------------------- */
    /* >>> Viewer Panel >>> */
    private Reference<Component> share_refPanel;
    private Reference<astex.MoleculeViewer> share_ref_astexMV;
    private UniqueList<ChAstex> share_vView;
    private JMenuBar share_menubar;
    private astex.MoleculeViewer getAstex() { return deref(share_ref_astexMV, astex.MoleculeViewer.class); }
    public ChAstex[] getViewersSharingView() { final UniqueList<ChAstex> v=this.share_vView; return v!=null ?  v.asArray() : null; }
    public Component getViewerPanel() { return share_refPanel!=null ? share_refPanel.get() : null; }
    /* <<< Viewer Panel <<< */
    /* ---------------------------------------- */
    /* >>> AWTEvent >>> */
    private final ChEventAdapter LI=new ChEventAdapter(this);
    public boolean processEvent(JComponent component, AWTEvent ev) {
        final int id=ev.getID();
        final astex.MoleculeRenderer mr=getMoleculeRenderer();
        final Protein p=getProtein();
        if (mr==null) return false;
        final int x=x(ev), y=y(ev);
        if (id==MouseEvent.MOUSE_CLICKED) {
            final int iA=toResidueIndex(mr.getNearestAtom(x, y));
            if (iA>=0) new ProteinEvent(this,ProteinEvent.PROTEIN_VIEWER_PICKED).setAminoAcid(iA,p).run();
        }
        mr.repaint();
        return true;
    }
    public JMenuBar getMenuBar() { return share_menubar; }
    public long getPropertyFlags() {
        return PROPERTY_MULTI_MOLECULE|PROPERTY_MULTI_VIEW
            |PROPERTY_HEAVY_WEIGHT|PROPERTY_COMMAND_INTERPRETER|PROPERTY_CAN_CENTER|PROPERTY_CAN_ZOOM|PROPERTY_CAN_ROTATE;
    }
    /* <<< AWTEvent <<< */
    /* ---------------------------------------- */
    /* >>> MoleculeRendererListener >>> */
    /* <<< MoleculeRendererListener <<< */
    /* ---------------------------------------- */
    /* >>> Matrix3D >>> */
    private final static WeakHashMap<astex.Molecule, float[]> mapMolecule2xyz=new WeakHashMap();
    public void transform(Matrix3D m3d){
        final astex.MoleculeRenderer mr=getMoleculeRenderer();
        final astex.Molecule molecule=mr!=null ? mr.getMolecule(this.proteinName) : null;
        if (molecule==null) return;
        float xyzORIG[]=mapMolecule2xyz.get(molecule);
        final int N=molecule.getAtomCount();
        if (xyzORIG==null) {
            if (m3d==null || m3d.isUnit()) return;
            mapMolecule2xyz.put(molecule,xyzORIG=new float[N*3]);
            for(int i=mini(N,xyzORIG.length/3); --i>=0;) {
                final astex.Atom atom=molecule.getAtom(i);
                for(int j=3;--j>=0;) xyzORIG[i*3+j]=(float)atom.get(j);
            }
        }
        assert xyzORIG.length==3*N;
        final float[] xyz;
        if (m3d==null || m3d.isUnit()) xyz=xyzORIG;
        else {
            xyz=new float[3*N];
            m3d.transformPoints(xyzORIG, xyz, N);
        }
        for(int i=mini(N,xyz.length/3); --i>=0; ) {
            final astex.Atom atom=molecule.getAtom(i);
            final int i3=3*i;
            atom.set(xyz[i3],xyz[i3+1],xyz[i3+2]);
        }
        interpret("object remove \"schematic_PROTEIN\";", (String)null);
        interpret(ChAstexPROXY.SCRIPT_SCHEMATIC,(String)null);
        interpret("center  ( sphere 20 around ( molecule '"+this.proteinName+"' ) ); ", (String)null);
        mr.repaint();
    }
    public void moleculeAdded(astex.MoleculeRenderer renderer, astex.Molecule molecule){}
    public void genericAdded(astex.MoleculeRenderer renderer, astex.generic.Generic generic){}
    public void genericRemoved(astex.MoleculeRenderer renderer, astex.generic.Generic generic){}
    public void mapAdded(astex.MoleculeRenderer renderer, astex.Map map){}
    public void mapRemoved(astex.MoleculeRenderer renderer, astex.Map map){}
    public void atomSelected(astex.MoleculeRenderer renderer, astex.Atom atom){
        sphere(atom);
    }
    public void moleculeRemoved(astex.MoleculeRenderer renderer, astex.Molecule molecule){
        //putln("MoleculeRendererListener moleculeRemoved");
        if (molecule!=null && ChAstex.this.proteinName.equals(molecule.getName())) ChAstex.this.dispose();
    }
}
/*
MoleculeRendererListener#atomSelected should be atomPicked
Atom#x  should be private
schematic should move with protein
remove group
how to list groups
how to draw cursor
*/
