package charite.christo.protein;
import charite.christo.*;
import charite.christo.interfaces.*;
import static charite.christo.ChUtils.*;
import static charite.christo.ChConstants.*;
import javax.swing.*;
import java.io.File;

/**HELP

OpenAstexViewer is distributed in Open Source form under the terms of
the LGPL license. Briefly, this means that you may distribute
OpenAstexViewer as part of other applications. You must display a
copyright notice that indicates that OpenAstexViewer is included in
your application.

See http://openastexviewer.net

   @author Christoph Gille

*/
public class  ChAstexPROXY extends AbstractProteinViewerProxy implements HasControlPanel {

    @Override public String getRequiredJars() { return "http://openastexviewer.net/web/OpenAstexViewer.jar"; }

    // java charite.christo.protein.ChAstexPROXY ~/m1/a1_SaccharomycesCerevisiae.pdb
    public static void main(String argv[]) throws Exception {
        final Protein p=new Protein();
        final File f=file(argv[0]);
        new PDB_Parser().parse(p,0,readBytes(f));
        p.setFile(f);
        final ChAstexPROXY pv=new ChAstexPROXY();
        pv.setProtein(p, null,  null, 0);
        new ChFrame("hello").ad(pv.getViewerPanel()).shw(0);
    }

    public final static String
        SCRIPT_SCHEMATIC="schematic -name 'schematic_PROTEIN'  ( molecule 'PROTEIN' );\n";
    public static String getCustomizableInitCommands() {
        return
            "secstruc ( molecule 'PROTEIN' );\n"+
            SCRIPT_SCHEMATIC+
            "display  lines off  ( molecule 'PROTEIN'  );\n"+
            "display lines on (all and aminoacid and ( atom C or atom CA or atom CB or atom N or atom O));\n"+
            "display sticks  on  (all and not aminoacid );\n"+
            "";
    }
    public static String getCustomizableLateCommands() {
        return "center (aminoacid );\n";
    }

    public static String getCustomizableExamples() {
        return
            "display  spheres on  (group RESIDUES);\n"+
            "display  spheres off (group RESIDUES);\n"+
            "display  cylinders on  (group RESIDUES);\n"+
            "display  cylinders off (group RESIDUES);\n"+
            "display  sticks on  (group RESIDUES);\n"+
            "display  sticks off  (group RESIDUES);\n"+
            "display  lines on (group RESIDUES);\n"+
            "display  lines off (group RESIDUES);\n"+
            "color green (group RESIDUES);\n"+
            "color red   (group RESIDUES);\n"+
            "color blue   (group RESIDUES);\n"+
            "color yellow   (group RESIDUES);\n"+
            "schematic -name 'schematic_RESIDUES' -alltube  true  -ribbonwidth 20 all;\n"+
            "object remove 'schematic_PROTEIN';\n"+
            "surface -solid true  'surface_RESIDUES' pink (all);\n"+
            "object remove 'surface_PROTEIN';\n"+
            SCRIPT_SCHEMATIC+
            "object remove 'schematic_PROTEIN';\n";
    }

    public static String getWordCompletionList() {
        return
            "select schematic  secstruc display center remove "+
            "object molecule aminoacid group surface current default "+
            "-alltube "+
            "-arrowheadwidth "+
            "-arrowsmoothing "+
            "-arrowthickness "+
            "-arrowwidth "+
            "-quality "+
            "-ribboncylinders "+
            "-ribbonellipse "+
            "-ribbonminwidth "+
            "-ribbonthickness "+
            "-ribbonwidth "+
            "-tuberadius "+
            "-tubesmoothing "+
            "-tubetaper "+
            "-tubetaperradius "+
            "true false ";
    }
}
