#
# USAGE
# ... speichern
#
# run *pfad*/mutagenesis.py
#
# load protein.pdb,prot
# select res,(resi 50 and chain X)
# mutate prot,res,TYR
#
#
# mutagenesis command for STRAP
# partly taken from PYMOL (c) Warren L. DeLano
#
# Warren L. DeLano, Kristian Rother, 2003
#

from pymol import cmd
from chempy import io

import pymol
import os
import string

obj_name = "mutat"
tmp_name = "tmp_mut"
tmp_obj1 = "tmp_obj1"
tmp_obj2 = "tmp_obj2"
tmp_obj3 = "tmp_obj3"

count = 0
cutoff = 3.5
   
library = io.pkl.fromFile(os.environ['PYMOL_PATH']+"/data/chempy/sidechains/sc_library.pkl")
      
residues = library.keys()
residues.extend(['GLY','PRO','ALA'])
residues.sort()

def mutate(object='pdb1cse',resi_obj='res',new_amino='LEU'):
   # auto zoom off
   auto_zoom = cmd.get_setting_text('auto_zoom')
   cmd.set('auto_zoom',"0",quiet=1)

   # get fragment from library
   cmd.feedback("push")
   cmd.feedback("disable","selector","everythin")
   cmd.feedback("disable","editor","actions")
      
   cmd.frame(0)
   if 1:
      res_type = new_amino
      cmd.fragment(string.lower(new_amino),tmp_name)
      cmd.remove("("+tmp_name+" and hydro)")
      # copy identifying information
      cmd.iterate("(%s and n;ca)"%resi_obj,"stored.chain=chain")
      cmd.alter("(%s)"%tmp_name,"chain=stored.chain")
      cmd.iterate("(%s and n;ca)"%resi_obj,"stored.resi=resi")
      cmd.alter("(%s)"%tmp_name,"resi=stored.resi")
      cmd.iterate("(%s and n;ca)"%resi_obj,"stored.segi=segi")
      cmd.alter("(%s)"%tmp_name,"segi=stored.segi")
      # move the fragment
      c1 = cmd.count_atoms("(%s and n;cb)"%tmp_name)
      c2 = cmd.count_atoms("(%s and n;cb)"%resi_obj)
      if (c1>0) and (c2>0):
         cmd.pair_fit("(%s and n;ca)"%tmp_name,
                      "(%s and n;ca)"%resi_obj,
                      "(%s and n;cb)"%tmp_name,
                      "(%s and n;cb)"%resi_obj,
                      "(%s and n;c)"%tmp_name,
                      "(%s and n;c)"%resi_obj,
                      "(%s and n;n)"%tmp_name,
                      "(%s and n;n)"%resi_obj)
      else:
         cmd.pair_fit("(%s and n;ca)"%tmp_name,
                      "(%s and n;ca)"%resi_obj,
                      "(%s and n;c)"%tmp_name,
                      "(%s and n;c)"%resi_obj,
                      "(%s and n;n)"%tmp_name,
                      "(%s and n;n)"%resi_obj)
      # fix the carbonyl position...
      cmd.iterate_state(1,"(%s and n;o)"%resi_obj,"stored.list=[x,y,z]")
      cmd.alter_state(1,"(%s and n;o)"%tmp_name,"(x,y,z)=stored.list")
      
   cmd.delete(obj_name)
   if library.has_key(res_type):
      lib = library[res_type][0]
      cmd.create(obj_name,tmp_name,1,1)
      for b in lib.keys():
         if b!='FREQ':
            cmd.set_dihedral("(%s & n;%s)"%(obj_name,b[0]),
                             "(%s & n;%s)"%(obj_name,b[1]),
                             "(%s & n;%s)"%(obj_name,b[2]),
                             "(%s & n;%s)"%(obj_name,b[3]),
                             lib[b],state=1)
      cmd.delete(tmp_name)
   else:
      cmd.create(obj_name,tmp_name,1,1)
   cmd.frame(0)
   cmd.feedback("pop")




   # create copy w/o residue
   cmd.do("create %s,(%s and not %s)"%(tmp_obj1,object,resi_obj))
   # save copy for bonded atom reference
   cmd.create(tmp_obj3,object)
   # cmd.delete(object)

   # create the merged molecule
   cmd.create(tmp_obj2,"(%s or %s)"%(tmp_obj1,obj_name),1) # only one state in merged object...
   
   # now connect them
   # 
   #
   #
   #cmd.do('select b1,(tmp_obj2 in mutat and n;N)') # OK   
   #cmd.do('select b2,(name C and (tmp_obj2 in (neighbor f91)))')
   #cmd.bond('b1','b2')
   cmd.bond("(%s in %s and n;N)"%(tmp_obj2,obj_name),
              "(name C and (%s in (neighbor %s)))"%(tmp_obj2,resi_obj))
   cmd.bond("(%s in %s and n;C)"%(tmp_obj2,obj_name),
              "(name N and (%s in (neighbor %s)))"%(tmp_obj2,resi_obj))
   
   # cmd.select(resi_obj,"(%s in %s)"%(object,resi_obj))
   cmd.delete(object)
   cmd.create(object,tmp_obj2)   

   cmd.delete(tmp_obj2)              
   cmd.delete(tmp_obj1)
   cmd.delete(tmp_obj3)
   cmd.delete(tmp_name)
   cmd.delete(obj_name)
   cmd.unpick()
#   cmd.show('ribbon')
               

cmd.extend('mutate',mutate)



